# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.forms import ValidationError
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from tacker_horizon.openstack_dashboard import api


class OnBoardNS(forms.SelfHandlingForm):
    name = forms.CharField(max_length=255, label=_("Name"))
    description = forms.CharField(widget=forms.widgets.Textarea(
                                  attrs={'rows': 4}),
                                  label=_("Description"),
                                  required=False)
    source_type = forms.ChoiceField(
        label=_('TOSCA Template Source'),
        required=False,
        choices=[('file', _('TOSCA Template File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'source'}))

    toscal_file = forms.FileField(
        label=_("TOSCA Template File"),
        help_text=_("A local TOSCA template file to upload."),
        widget=forms.FileInput(
            attrs={'class': 'switched', 'data-switch-on': 'source',
                   'data-source-file': _('TOSCA Template File')}),
        required=False)

    direct_input = forms.CharField(
        label=_('TOSCA YAML'),
        help_text=_('The YAML formatted contents of a TOSCA template.'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched', 'data-switch-on': 'source',
                   'data-source-raw': _('TOSCA YAML')}),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(OnBoardNS, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(OnBoardNS, self).clean()

        # The key can be missing based on particular upload
        # conditions. Code defensively for it here...
        toscal_file = data.get('toscal_file', None)
        toscal_raw = data.get('direct_input', None)
        source_type = data.get("source_type")
        if source_type == "file" and not toscal_file:
            raise ValidationError(
                _("No TOSCA template file selected."))
        if source_type == "raw" and not toscal_raw:
            raise ValidationError(
                _("No direct input specified."))

        if toscal_file and not toscal_file.name.endswith(('.yaml', '.csar')):
            raise ValidationError(_("Only .yaml or .csar file uploads \
                                    are supported"))

        try:
            if toscal_file:
                toscal_str = self.files['toscal_file'].read()
            else:
                toscal_str = data['direct_input']
            # toscal = yaml.loads(toscal_str)
            data['tosca'] = toscal_str
        except Exception as e:
            msg = _('There was a problem loading the namespace: %s.') % e
            raise forms.ValidationError(msg)

        return data

    def handle(self, request, data):
        try:
            toscal = data['tosca']
            nsd_name = data['name']
            nsd_description = data['description']
            tosca_arg = {'nsd': {'name': nsd_name,
                                 'description': nsd_description,
                                 'attributes': {'nsd': toscal}}}
            nsd_instance = api.tacker.create_nsd(request, tosca_arg)
            messages.success(request,
                             _('NS Catalog entry %s has been created.') %
                             nsd_instance['nsd']['name'])
            return toscal
        except Exception:
            msg = _('Unable to create TOSCA.')
            exceptions.handle(request, msg)
            return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffgmanager \
    import tables
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffgmanager.tables \
    import VNFFGManagerItem
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffgmanager.tables \
    import VNFFGManagerItemList


class VNFFGManagerTab(tabs.TableTab):
    name = _("VNFFGManager Tab")
    slug = "vnffgmanager_tab"
    table_classes = (tables.VNFFGManagerTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_vnffgmanager_data(self):
        try:
            VNFFGManagerItemList.clear_list()
            vnffgs = api.tacker.vnffg_list(self.request)

            if len(vnffgs) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for vnffg in vnffgs:
                try:
                    vnffg_desc_str = vnffg['description']
                except KeyError:
                    vnffg_desc_str = ""

                obj = VNFFGManagerItem(vnffg['id'],
                                       vnffg['name'],
                                       vnffg_desc_str,
                                       vnffg['status'])
                VNFFGManagerItemList.add_item(obj)
            return VNFFGManagerItemList.VNFFGLIST_P
        except Exception:
            self._has_more = False
            error_message = _('Unable to get instances')
            exceptions.handle(self.request, error_message)

            return []


class VNFFGManagerTabs(tabs.TabGroup):
    slug = "vnffgmanager_tabs"
    tabs = (VNFFGManagerTab,)
    sticky = True


class VNFFGDetailsTab(tabs.Tab):
    name = _("VNFFG Detail")
    slug = "VNFFG_Details"
    template_name = "nfv/vnffgmanager/vnffg_details.html"

    def get_context_data(self, request):
        return {'vnffg': self.tab_group.kwargs['vnffg']}


class VNFFGDetailsTabs(tabs.TabGroup):
    slug = "VNFFG_details"
    tabs = (VNFFGDetailsTab,)
    sticky = True



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfpmthreshold \
    import tables


class VnfPmThresholdItem(object):
    def __init__(self, threshold_id, links, object_type):
        self.id = threshold_id
        self.name = threshold_id
        self.links = links
        self.object_type = object_type


class VnfPmThresholdTab(tabs.TableTab):
    name = _("VNFPMThreshold Tab")
    slug = "vnfpmthreshold_tab"
    table_classes = (tables.VnfPmThresholdTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = False

    def has_prev_data(self, table):
        return self._has_prev

    def has_more_data(self, table):
        return self._has_more

    def get_pmthreshold_data(self):
        try:
            self._has_prev = False
            page_size = horizon_utils.functions.get_page_size(self.request)
            marker = self.request.GET.get("vnfpmthreshold_marker", None)
            prev_marker = self.request.GET.get(
                "prev_vnfpmthreshold_marker", None)
            pmthresholds = api.tacker.list_pm_thresholds(self.request)

            if marker is not None or prev_marker is not None:
                for i, threshold in enumerate(pmthresholds):
                    if threshold["id"] == marker and i < len(pmthresholds) - 1:
                        pmthresholds = pmthresholds[i + 1:]
                        self._has_prev = True
                        break
                    if threshold["id"] == prev_marker and i > page_size:
                        pmthresholds = pmthresholds[i - page_size:]
                        self._has_prev = True
                        break

            if len(pmthresholds) > page_size:
                self._has_more = True
            else:
                self._has_more = False

            rows = []
            for i, threshold in enumerate(pmthresholds):
                if i >= page_size:
                    break
                item = VnfPmThresholdItem(threshold['id'],
                                          threshold['_links'],
                                          threshold['objectType'])
                rows.append(item)
            return rows
        except Exception:
            self._has_more = False
            error_message = _('Failed to get PM Thresholds.')
            exceptions.handle(self.request, error_message)
            return []


class VnfPmThresholdTabs(tabs.TabGroup):
    slug = "vnfpmthreshold_tabs"
    tabs = (VnfPmThresholdTab,)
    sticky = True


class VnfPmThresholdDetailTab(tabs.Tab):
    name = _("VNF PM Threshold")
    slug = "vnfpmthreshold_detail_tab"
    template_name = "nfv/vnfpmthreshold/pmthreshold_detail.html"

    def get_context_data(self, request):
        return {'pmthreshold': self.tab_group.kwargs['pmthreshold']}


class VnfPmThresholdDetailTabs(tabs.TabGroup):
    slug = "vnfpmthreshold_detail_tabs"
    tabs = (VnfPmThresholdDetailTab,)
    sticky = True



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfpmjob \
    import forms as project_forms
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfpmjob \
    import tabs as pmjob_tabs


class IndexView(tabs.TabbedTableView):
    # A very simple class-based view...
    tab_group_class = pmjob_tabs.VnfPmJobTabs
    template_name = 'nfv/vnfpmjob/index.html'

    def get_data(self, request, context, *args, **kwargs):
        # Add data to the context here...
        return context


class CreatePmJobView(forms.ModalFormView):
    form_class = project_forms.CreatePmJob
    template_name = 'nfv/vnfpmjob/create_pmjob.html'
    success_url = reverse_lazy("horizon:nfv:vnfpmjob:index")
    modal_id = "create_pmjob_modal"
    modal_header = _("Create PM Job")
    submit_label = _("Create PM Job")
    submit_url = "horizon:nfv:vnfpmjob:createpmjob"

    def get_context_data(self, **kwargs):
        context = super(CreatePmJobView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url)
        return context


class UpdatePmJobView(forms.ModalFormView):
    form_class = project_forms.UpdatePmJob
    template_name = 'nfv/vnfpmjob/update_pmjob.html'
    success_url = reverse_lazy("horizon:nfv:vnfpmjob:index")
    modal_id = "update_pmjob_modal"
    modal_header = _("Update PM Job")
    submit_label = _("Update PM Job")
    submit_url = "horizon:nfv:vnfpmjob:updatepmjob"

    def get_context_data(self, **kwargs):
        context = super(UpdatePmJobView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class DetailView(tabs.TabView):
    tab_group_class = pmjob_tabs.VnfPmJobDetailTabs
    template_name = 'nfv/vnfpmjob/detail.html'
    redirect_url = 'horizon:nfv:vnfpmjob:index'
    page_title = _("PM Job Detail: {{ id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        pmjob = self.get_data()
        context['pmjob'] = pmjob
        context['id'] = kwargs['id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        job_id = self.kwargs['id']

        try:
            pmjob = tacker_api.tacker.get_pm_job(self.request, job_id)
            pmjob['links'] = pmjob.get('_links', '')
            pmjob['callback_uri'] = pmjob.get('callbackUri', '')
            pmjob['criteria'] = pmjob.get('criteria', '')
            pmjob['id'] = pmjob.get('id', '')
            pmjob['object_instance_ids'] = pmjob.get('objectInstanceIds', '')
            pmjob['object_type'] = pmjob.get('objectType', '')
            pmjob['sub_object_instance_ids'] = pmjob.get(
                                                'subObjectInstanceIds', '')
            pmjob['reports'] = self.add_reportid(pmjob.get('reports', ''))

            return pmjob
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Failed to get PM Job. (id: %s)') %
                              job_id, redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        pmjob = self.get_data()
        return self.tab_group_class(request, pmjob=pmjob, **kwargs)

    def add_reportid(self, reports=None):
        repots_new = []
        for report in reports:
            href = report.get('href', '')
            target_reports = 'reports/'
            target_jobs = 'pm_jobs/'
            idx_reports = href.find(target_reports)
            idx_jobs = href.find(target_jobs)

            if (0 <= idx_reports) and (0 <= idx_jobs):
                report['reportId'] = href[idx_reports+len(target_reports):]
                report['id'] = href[idx_jobs+len(target_jobs):idx_reports-1]

            repots_new.extend([report])
        return repots_new


class ReportDetailView(tabs.TabView):
    tab_group_class = pmjob_tabs.VnfPmJobReportDetailTabs
    template_name = 'nfv/vnfpmjob/detail.html'
    redirect_url = 'horizon:nfv:vnfpmjob:index'
    page_title = _("PM Job Report: {{ reportId }}")
    href_url = 'horizon:nfv:vnfpmjob:reportdetail'

    def get_context_data(self, **kwargs):
        context = super(ReportDetailView, self).get_context_data(**kwargs)
        report = self.get_data()
        context['report'] = report
        context['id'] = kwargs['id']
        context['reportId'] = kwargs['reportId']
        context['url'] = reverse(self.redirect_url)
        context['submit_url'] = reverse(
                self.href_url, kwargs={'id': self.kwargs['id'],
                                       'reportId': self.kwargs['reportId']})
        return context

    @memoized.memoized_method
    def get_data(self):

        job_id = self.kwargs['id']
        report_id = self.kwargs['reportId']

        try:
            report = tacker_api.tacker.get_pm_report(self.request,
                                                     job_id, report_id)
            report['entries'] = report.get('entries', '')
            return report
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Failed to get Report. (id: %s)') % report_id,
                              redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        report = self.get_data()
        return self.tab_group_class(request, report=report, **kwargs)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django import http
from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfpackages \
    import forms as project_forms
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfpackages \
    import tabs as vnfpkg_tabs


def fetch_vnf_package(request, id):
    try:
        file = tacker_api.tacker.fetch_vnf_package(request, id)

    except Exception:
        exceptions.handle(request,
                          _('Failed to fetch VNF Package. (id: %s)') % id,
                          redirect="horizon:nfv:vnfpackages:index")
    response = http.HttpResponse(content_type='application/zip')
    response.write(file)
    response['Content-Disposition'] = (
        'attachment; filename="package-%s.zip"' % id)
    response['Content-Length'] = len(file)

    return response


class IndexView(tabs.TabbedTableView):
    tab_group_class = vnfpkg_tabs.VnfPackageTabs
    template_name = 'nfv/vnfpackages/index.html'


class UploadVnfPackageView(forms.ModalFormView):
    form_class = project_forms.UploadVnfPackage
    template_name = 'nfv/vnfpackages/upload_vnfpkg.html'
    success_url = reverse_lazy("horizon:nfv:vnfpackages:index")
    modal_id = "add_service_modal"
    modal_header = _("Upload VNF Package")
    submit_label = _("Upload VNF Package")
    submit_url = "horizon:nfv:vnfpackages:uploadvnfpkg"

    def get_context_data(self, **kwargs):
        context = super(UploadVnfPackageView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url)
        return context


class UpdateVnfPackageView(forms.ModalFormView):
    form_class = project_forms.UpdateVnfPackage
    template_name = 'nfv/vnfpackages/update_vnfpkg.html'
    success_url = reverse_lazy("horizon:nfv:vnfpackages:index")
    modal_id = "add_service_modal"
    modal_header = _("Update VNF Package Info")
    submit_label = _("Update VNF Package Info")
    submit_url = "horizon:nfv:vnfpackages:updatevnfpkg"

    def get_context_data(self, **kwargs):
        context = super(UpdateVnfPackageView, self).get_context_data(**kwargs)
        context['id'] = self.kwargs['id']
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class DetailView(tabs.TabView):
    tab_group_class = vnfpkg_tabs.VnfPackageDetailTabs
    template_name = 'nfv/vnfpackages/detail.html'
    redirect_url = 'horizon:nfv:vnfpackages:index'
    page_title = _("VNF Package Details: {{ id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        vnfpkg = self.get_data()
        context['vnfpkg'] = vnfpkg
        context['id'] = kwargs['id']
        context['url'] = reverse(self.redirect_url)

        return context

    @memoized.memoized_method
    def get_data(self):
        vnfpkg_id = self.kwargs.get('id', None)

        try:
            vnfpkg = tacker_api.tacker.get_vnf_package(self.request, vnfpkg_id)
            vnfpkg['links'] = vnfpkg.get('_links', '')
            return vnfpkg
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(
                self.request,
                _('Failed to get VNF Package. (id: %s)') % vnfpkg_id,
                redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        vnfpkg = self.get_data()
        return self.tab_group_class(request, vnfpkg=vnfpkg, **kwargs)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from openstack_dashboard.test.integration_tests import helpers


class TestTackerDashboardInstalled(helpers.TestCase):

    def test_vnf_catalog_page_opened(self):
        vnf_catalog = self.home_pg.go_to_nfv_vnfmanagement_vnfcatalogpage()
        self.assertEqual(vnf_catalog.page_title,
                         'VNF Catalog - OpenStack Dashboard')

    def test_vnf_manager_page_opened(self):
        vnf_manager = self.home_pg.go_to_nfv_vnfmanagement_vnfmanagerpage()
        self.assertEqual(vnf_manager.page_title,
                         'VNF Manager - OpenStack Dashboard')

    def test_vim_page_opened(self):
        vim = self.home_pg.go_to_nfv_nfvorchestration_vimmanagementpage()
        self.assertEqual(vim.page_title,
                         'VIM - OpenStack Dashboard')

    def test_vnffg_catalog_page_opened(self):
        vnffg_catalog = (
                self.home_pg.go_to_nfv_nfvorchestration_vnffgcatalogpage())
        self.assertEqual(vnffg_catalog.page_title,
                         'VNFFG Catalog - OpenStack Dashboard')

    def test_vnffg_manager_page_opened(self):
        vnffg_manager = (
                self.home_pg.go_to_nfv_nfvorchestration_vnffgmanagerpage())
        self.assertEqual(vnffg_manager.page_title,
                         'VNFFG Manager - OpenStack Dashboard')

    def test_ns_catalog_page_opened(self):
        ns_catalog = self.home_pg.go_to_nfv_nfvorchestration_nscatalogpage()
        self.assertEqual(ns_catalog.page_title,
                         'NS Catalog - OpenStack Dashboard')

    def test_ns_manager_page_opened(self):
        ns_manager = self.home_pg.go_to_nfv_nfvorchestration_nsmanagerpage()
        self.assertEqual(ns_manager.page_title,
                         'NS Manager - OpenStack Dashboard')

    def test_vnf_packages_page_opened(self):
        vnf_packages = self.home_pg.go_to_nfv_vnfpackages_vnfpackagespage()
        self.assertEqual(vnf_packages.page_title,
                         'VNF Packages - OpenStack Dashboard')

    def test_vnflcm_page_opened(self):
        vnflcm = self.home_pg.go_to_nfv_vnflcm_vnflcmpage()
        self.assertEqual(vnflcm.page_title,
                         'VNF LCM - OpenStack Dashboard')

    def test_vnflcm_opocc_page_opened(self):
        vnflcm_opocc = self.home_pg.go_to_nfv_vnflcm_vnflcmopoccpage()
        self.assertEqual(vnflcm_opocc.page_title,
                         'VNF LCM OP OCC - OpenStack Dashboard')

    def test_vnflcm_subscription_page_opened(self):
        vnflcm_subsc = self.home_pg.go_to_nfv_vnflcm_lccnsubscriptionpage()
        self.assertEqual(vnflcm_subsc.page_title,
                         'LCCN Subscription - OpenStack Dashboard')

    def test_vnffm_alarm_page_opened(self):
        fm_alarm = self.home_pg.go_to_nfv_vnffm_vnffmalarmpage()
        self.assertEqual(fm_alarm.page_title,
                         'Alarm - OpenStack Dashboard')

    def test_vnffm_subscription_page_opened(self):
        fm_subsc = self.home_pg.go_to_nfv_vnffm_vnffmsubscriptionpage()
        self.assertEqual(fm_subsc.page_title,
                         'Subscription - OpenStack Dashboard')

    def test_vnfpm_pmjob_page_opened(self):
        pm_job = self.home_pg.go_to_nfv_vnfpm_vnfpmjobpage()
        self.assertEqual(pm_job.page_title,
                         'PM Job - OpenStack Dashboard')

    def test_vnfpm_pmthreshold_page_opened(self):
        pm_threshold = self.home_pg.go_to_nfv_vnfpm_vnfpmthresholdpage()
        self.assertEqual(pm_threshold.page_title,
                         'PM Threshold - OpenStack Dashboard')



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import tables
from openstack_dashboard import policy

from tacker_horizon.openstack_dashboard import api


class DeletePmJob(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete PM Job",
            "Delete PM Jobs",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Delete PM Job",
            "Delete PM Jobs",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_pm_job(request, obj_id)


class CreatePmJob(tables.LinkAction):
    name = "createpmjob"
    verbose_name = _("Create PM Job")
    classes = ("ajax-modal",)
    icon = "plus"
    url = "horizon:nfv:vnfpmjob:createpmjob"


class UpdatePmJob(tables.LinkAction):
    name = "updatepmjob"
    verbose_name = _("Update PM Job")
    url = "horizon:nfv:vnfpmjob:updatepmjob"
    classes = ("ajax-modal",)


class VnfPmJobTable(tables.DataTable):
    id = tables.Column('id', link="horizon:nfv:vnfpmjob:detail",
                       verbose_name=_("ID"))
    object_type = tables.Column('object_type', verbose_name=_("Object Type"))
    links = tables.Column('links', verbose_name=_("Links"))

    class Meta(object):
        name = "pmjob"
        verbose_name = _("PM Job")
        pagination_param = 'vnfpmjob_marker'
        prev_pagination_param = 'prev_vnfpmjob_marker'
        table_actions = (CreatePmJob, DeletePmJob, tables.FilterAction,)
        row_actions = (UpdatePmJob, DeletePmJob,)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import tables

from tacker_horizon.openstack_dashboard import api


class VnfLcmRollback(tables.BatchAction):
    name = "vnflcmrollback"
    verbose_name = _("Rollback VNF Lifecycle Management Operation")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Rollback VNF Lifecycle Management Operation",
            "Rollback VNF Lifecycle Management Operation",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Rollback VNF Lifecycle Management Operation",
            "Rollback VNF Lifecycle Management Operation",
            count
        )

    def action(self, request, obj_id):
        api.tacker.rollback_vnf_lcm_op_occ(request, obj_id)


class VnfLcmRetry(tables.BatchAction):
    name = "vnflcmretry"
    verbose_name = _("Retry")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Retry",
            "Retry",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Retry",
            "Retry",
            count
        )

    def action(self, request, obj_id):
        api.tacker.retry_vnf_lcm_op_occ(request, obj_id)


class VnfLcmFail(tables.BatchAction):
    name = "vnflcmfail"
    verbose_name = _("Fail")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Fail",
            "Fail",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Fail",
            "Fail",
            count
        )

    def action(self, request, obj_id):
        api.tacker.fail_vnf_lcm_op_occ(request, obj_id)


class VnfLcmOpOccTable(tables.DataTable):
    id = tables.Column('id', verbose_name=_("ID"),
                       link="horizon:nfv:vnflcmopocc:detail",)
    operation_state = tables.Column('operation_state',
                                    verbose_name=_("OperationState"))
    vnf_instance_id = tables.Column('vnf_instance_id',
                                    verbose_name=_("VNFInstanceID"))
    operation = tables.Column('operation', verbose_name=_("Operation"))

    class Meta(object):
        name = "vnflcmopocc"
        verbose_name = _("VNF LCM OP OCC")
        pagination_param = 'opocc_marker'
        prev_pagination_param = 'prev_opocc_marker'
        table_actions = (VnfLcmRollback, VnfLcmRetry, VnfLcmFail,
                         tables.FilterAction,)
        row_actions = (VnfLcmRollback, VnfLcmRetry, VnfLcmFail,)
        multi_select = True



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import tables
from openstack_dashboard import policy

from tacker_horizon.openstack_dashboard import api


class DeleteLccnSubscription(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete Lccn Subscription",
            "Delete Lccn Subscriptions",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Delete Lccn Subscription",
            "Delete Lccn Subscriptions",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_vnf_lcm_subscription(request, obj_id)


class CreateLccnSubscription(tables.LinkAction):
    name = "create_lccnsubsc"
    verbose_name = _("Create Lccn Subscription")
    classes = ("ajax-modal",)
    icon = "plus"
    url = "horizon:nfv:lccnsubscription:createlccnsubscription"


class LccnSubscriptionTable(tables.DataTable):
    id = tables.Column('id', verbose_name=_("ID"),
                       link="horizon:nfv:lccnsubscription:detail",)
    callback_uri = tables.Column('callback_uri',
                                 verbose_name=_("Callback URI"))

    class Meta(object):
        name = "lccnsubsc"
        verbose_name = _("LCCN Subscription")
        pagination_param = 'subsc_marker'
        prev_pagination_param = 'prev_subsc_marker'
        table_actions = (CreateLccnSubscription, DeleteLccnSubscription,
                         tables.FilterAction)
        row_actions = (DeleteLccnSubscription,)
        multi_select = True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv.nsmanager import tables
from tacker_horizon.openstack_dashboard.dashboards.nfv import utils


class NSManagerTab(tabs.TableTab):
    name = _("NSManager Tab")
    slug = "nsmanager_tab"
    table_classes = (tables.NSManagerTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_nsmanager_data(self):
        try:
            tables.NSManagerItemList.clear_list()
            nss = api.tacker.ns_list(self.request)

            if len(nss) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for ns in nss:
                try:
                    ns_desc_str = ns['description']
                except KeyError:
                    ns_desc_str = ""

                vim = ns['vim_id']
                obj = tables.NSManagerItem(
                    ns['name'],
                    ns_desc_str,
                    vim,
                    ns['status'],
                    ns['id'],
                    ns['error_reason'])
                tables.NSManagerItemList.add_item(obj)
            return tables.NSManagerItemList.NSLIST_P
        except Exception:
            self._has_more = False
            error_message = _('Unable to get instances')
            exceptions.handle(self.request, error_message)

            return []


class NSManagerTabs(tabs.TabGroup):
    slug = "nsmanager_tabs"
    tabs = (NSManagerTab,)
    sticky = True


class NSEventsTab(tabs.TableTab):
    name = _("Events Tab")
    slug = "events_tab"
    table_classes = (utils.EventsTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_more_data(self, table):
        return self._has_more

    def get_events_data(self):
        try:
            utils.EventItemList.clear_list()
            events = api.tacker.events_list(self.request,
                                            self.tab_group.kwargs['ns_id'])

            if len(events) > horizon_utils.functions.get_page_size(
                    self.request):
                self._has_more = True
            else:
                self._has_more = False

            for event in events:
                evt_obj = utils.EventItem(
                    event['id'], event['resource_state'],
                    event['event_type'],
                    event['timestamp'],
                    event['event_details'])
                utils.EventItemList.add_item(evt_obj)
            return utils.EventItemList.EVTLIST_P
        except Exception:
            self._has_more = False
            error_message = _('Unable to get events.')
            exceptions.handle(self.request, error_message)
            return []


class NSDetailsTabs(tabs.TabGroup):
    slug = "NS_details"
    tabs = (NSEventsTab,)
    sticky = True



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import time

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from tacker_horizon.openstack_dashboard import api


class CreateVnfIdentifier(forms.SelfHandlingForm):
    vnfd_id = forms.CharField(label=_("VNFD ID"))
    param_file = forms.FileField(
       label=_("Param File"),
       widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-upload': _('Param File')}),
       required=False)
    name = forms.CharField(label=_("Name"), required=False)
    description = forms.CharField(label=_("Description"), required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateVnfIdentifier, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(CreateVnfIdentifier, self).clean()

        param_file = data.get('param_file', None)

        try:
            if param_file:
                metadata = self.files['param_file'].read()
                data['metadata'] = json.loads(metadata)
        except Exception as e:
            msg = _('Failed to read file: %s.') % e
            raise forms.ValidationError(msg)
        return data

    def handle(self, request, data):
        try:
            vnfd_id = data.get('vnfd_id', None)
            metadata = data.get('metadata', None)
            name = data.get('name', None)
            description = data.get('description', None)

            body = {}
            body['vnfdId'] = vnfd_id
            if metadata:
                body['metadata'] = metadata
            if name:
                body['vnfInstanceName'] = name
            if description:
                body['vnfInstanceDescription'] = description

            response = api.tacker.create_vnf_instance(request, body)
            messages.success(request,
                             _('Create VNF Identifier. (id: %s)') %
                             response['id'])

        except Exception:
            exceptions.handle(request, _('Failed to create VNF Identifier.'))
            return False

        return True


class InstantiateVnf(forms.SelfHandlingForm):
    param_file = forms.FileField(
       label=_("Param File"),
       widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-upload': _('Param File')}))

    def __init__(self, request, *args, **kwargs):
        super(InstantiateVnf, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(InstantiateVnf, self).clean()

        try:
            param_str = self.files['param_file'].read()
            param = json.loads(param_str)
            data['param_data'] = param
        except Exception as e:
            msg = _('Failed to read file: %s.') % e
            raise forms.ValidationError(msg)

        return data

    def handle(self, request, data):
        try:
            vnf_id = request.resolver_match.kwargs.get('id', None)
            param = data['param_data']
            api.tacker.instantiate_vnf_instance(request, vnf_id, param)
            messages.success(request,
                             _('Accepted to instantiate VNF Instance. '
                               '(id: %s)') % vnf_id)

        except Exception:
            exceptions.handle(request,
                              _('Failed to instantiate VNF Instance. '
                                '(id: %s)') % vnf_id)
            return False

        return True


class TerminateVnf(forms.SelfHandlingForm):
    termination_type = forms.ChoiceField(
        label=_('Termination Type'),
        required=True,
        choices=[('GRACEFUL', _('GRACEFUL')),
                 ('FORCEFUL', _('FORCEFUL'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'source'}))
    termination_timeout = forms.IntegerField(
        min_value=0,
        required=False,
        label=_('Graceful Termination Timeout'))
    is_delete = forms.BooleanField(
        label=_('Delete VNF Instance'),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(TerminateVnf, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(TerminateVnf, self).clean()
        return data

    def handle(self, request, data):
        try:
            is_delete = data.get('is_delete', False)
            vnf_id = request.resolver_match.kwargs.get('id', None)
            body = {}
            termination_type = data.get('termination_type', None)
            body['terminationType'] = termination_type
            termination_timeout = data.get('terminationTimeout', None)
            if termination_timeout:
                body['gracefulTerminationTimeout'] = termination_timeout
            api.tacker.terminate_vnf_instance(request, vnf_id, body)
            if not is_delete:
                messages.success(request,
                                 _('Accepted to terminate VNF Instance. '
                                   '(id: %s)') % vnf_id)
        except Exception:
            exceptions.handle(request,
                              _('Failed to terminate VNF Instance. '
                                '(id: %s)') % vnf_id)
            return False

        if is_delete:
            retry = 0
            retry_limit = 12
            while True:
                try:
                    time.sleep(10)
                    api.tacker.delete_vnf_instance(request, vnf_id)
                    messages.success(request,
                                     _('Delete VNF Identifier. (id: %s)') %
                                     vnf_id)
                    break
                except Exception as exc:
                    if 'is in progress' in str(exc):
                        # Conflict error occurs even if delete
                        # is executed immediately after terminate
                        retry = retry + 1
                        if retry >= retry_limit:
                            exceptions.handle(
                                request,
                                _('Delete VNF Identifier retry out. (id: %s)')
                                % vnf_id)
                            break
                        continue
                    else:
                        exceptions.handle(
                            request,
                            _('Failed to delete VNF Identifer. (id: %s)') %
                            vnf_id)
                        break
        return True


class HealVnf(forms.SelfHandlingForm):
    cause = forms.CharField(label=_("Cause"),
                            required=False)
    vnfc_instances = forms.CharField(
        label=_('VNFC Instance'), required=False, widget=forms.Textarea,
        help_text=_('Enter VNFC instance IDs in comma-separated list format.'))

    def __init__(self, request, *args, **kwargs):
        super(HealVnf, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(HealVnf, self).clean()
        return data

    def handle(self, request, data):
        try:
            vnf_id = request.resolver_match.kwargs.get('id', None)
            body = {}
            cause = data.get('cause', None)
            if cause:
                body['cause'] = cause
            array_str = data.get('vnfc_instances', None)
            if array_str:
                array_str.replace('\n', '')
                vnfc_instance_ids = [s.strip() for s in array_str.split(',')]
                body['vnfcInstanceId'] = vnfc_instance_ids
            api.tacker.heal_vnf_instance(request, vnf_id, body)
            messages.success(request,
                             _('Accepted to heal VNF Instance. (id: %s)') %
                             vnf_id)

        except Exception:
            exceptions.handle(request,
                              _('Failed to heal VNF Instance. (id: %s)') %
                              vnf_id)
            return False

        return True


class UpdateVnf(forms.SelfHandlingForm):
    param_file = forms.FileField(
       label=_("Param File"),
       widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-upload': _('Param File')}))

    def __init__(self, request, *args, **kwargs):
        super(UpdateVnf, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(UpdateVnf, self).clean()
        try:
            param_str = self.files['param_file'].read()
            param = json.loads(param_str)
            data['param_data'] = param
        except Exception as e:
            msg = _('Failed to read file: %s.') % e
            raise forms.ValidationError(msg)

        return data

    def handle(self, request, data):
        try:
            vnf_id = request.resolver_match.kwargs.get('id', None)
            param = data['param_data']
            api.tacker.update_vnf_instance(request, vnf_id, param)
            messages.success(request,
                             _('Accepted to update VNF Instance. '
                               '(id: %s)') % vnf_id)

        except Exception:
            exceptions.handle(request,
                              _('Failed to update VNF Instance. '
                                '(id: %s)') % vnf_id)
            return False

        return True


class ScaleVnf(forms.SelfHandlingForm):
    scale_type = forms.ChoiceField(
        label=_('Type'),
        choices=[('SCALE_IN', _('SCALE_IN')),
                 ('SCALE_OUT', _('SCALE_OUT'))])
    aspect_id = forms.CharField(label=_("Aspect ID"))
    num_of_steps = forms.IntegerField(
        label=_("Number of Steps"), min_value=1, required=False)
    param_file = forms.FileField(
        label=_("Param File"),
        widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-upload': _('Param File')}),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(ScaleVnf, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(ScaleVnf, self).clean()

        param_file = data.get('param_file', None)

        try:
            if param_file:
                additional_params = self.files['param_file'].read()
                data['additional_params'] = json.loads(additional_params)
        except Exception as e:
            msg = _('Failed to read file: %s.') % e
            raise forms.ValidationError(msg)
        return data

    def handle(self, request, data):
        try:
            vnf_id = request.resolver_match.kwargs.get('id', None)
            scale_type = data.get('scale_type', None)
            aspect_id = data.get('aspect_id', None)
            num_of_steps = data.get('num_of_steps', None)
            additional_params = data.get('additional_params', None)

            body = {}
            body['type'] = scale_type
            body['aspectId'] = aspect_id
            if num_of_steps:
                body['numberOfSteps'] = num_of_steps
            if additional_params:
                body['additionalParams'] = additional_params

            api.tacker.scale_vnf_instance(request, vnf_id, body)
            messages.success(request,
                             _('Accepted to scale VNF Instance. '
                               '(id: %s)') % vnf_id)

        except Exception:
            exceptions.handle(request,
                              _('Failed to scale VNF Instance. '
                                '(id: %s)') % vnf_id)
            return False

        return True


class ChangeExternalVnfConnectivity(forms.SelfHandlingForm):
    param_file = forms.FileField(
       label=_("Param File"),
       widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-upload': _('Param File')}))

    def __init__(self, request, *args, **kwargs):
        super(ChangeExternalVnfConnectivity,
              self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(ChangeExternalVnfConnectivity, self).clean()
        try:
            param_str = self.files['param_file'].read()
            param = json.loads(param_str)
            data['param_data'] = param
        except Exception as e:
            msg = _('Failed to read file: %s.') % e
            raise forms.ValidationError(msg)
        return data

    def handle(self, request, data):
        try:
            vnf_id = request.resolver_match.kwargs.get('id', None)
            param = data['param_data']
            api.tacker.change_ext_conn_vnf_instance(request, vnf_id, param)
            messages.success(request,
                             _('Accepted to change External VNF Connectivity. '
                               '(id: %s)') % vnf_id)

        except Exception:
            exceptions.handle(request,
                              _('Failed to change External VNF Connectivity. '
                                '(id: %s)') % vnf_id)
            return False

        return True


class ChangeCurrentVnfPackage(forms.SelfHandlingForm):
    param_file = forms.FileField(
       label=_("Param File"),
       widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-upload': _('Param File')}))

    def __init__(self, request, *args, **kwargs):
        super(ChangeCurrentVnfPackage, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(ChangeCurrentVnfPackage, self).clean()
        try:
            param_str = self.files['param_file'].read()
            param = json.loads(param_str)
            data['param_data'] = param
        except Exception as e:
            msg = _('Failed to read file: %s.') % e
            raise forms.ValidationError(msg)
        return data

    def handle(self, request, data):
        try:
            vnf_id = request.resolver_match.kwargs.get('id', None)
            param = data['param_data']
            api.tacker.change_vnfpkg_vnf_instance(request, vnf_id, param)
            messages.success(request,
                             _('Accepted to change Current VNF Package. '
                               '(id: %s)') % vnf_id)

        except Exception:
            exceptions.handle(request,
                              _('Failed to change Current VNF Package. '
                                '(id: %s)') % vnf_id)
            return False

        return True



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.lccnsubscription \
    import forms as project_forms
from tacker_horizon.openstack_dashboard.dashboards.nfv.lccnsubscription \
    import tabs as lccnsubsctabs


class IndexView(tabs.TabbedTableView):
    tab_group_class = lccnsubsctabs.LccnSubscriptionTabs
    template_name = 'nfv/lccnsubscription/index.html'


class CreateLccnSubscriptionView(forms.ModalFormView):
    form_class = project_forms.CreateLccnSubscription
    template_name = 'nfv/lccnsubscription/create_lccn_subscription.html'
    success_url = reverse_lazy("horizon:nfv:lccnsubscription:index")
    modal_id = "add_service_modal"
    modal_header = _("Create Lccn Subscription")
    submit_label = _("Create Lccn Subscription")
    submit_url = "horizon:nfv:lccnsubscription:createlccnsubscription"

    def get_context_data(self, **kwargs):
        context = super(
            CreateLccnSubscriptionView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url)
        return context


class DetailView(tabs.TabView):
    tab_group_class = lccnsubsctabs.LccnSubscDetailTabs
    template_name = 'nfv/lccnsubscription/detail.html'
    redirect_url = 'horizon:nfv:lccnsubscription:index'
    page_title = _("LCCN Subscription Detail")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        lccnsubscription = self.get_data()
        context['lccnsubscription'] = lccnsubscription
        context['id'] = kwargs['id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        subsc_id = self.kwargs.get('id', None)

        try:
            lccnsubscription = tacker_api.tacker.get_vnf_lcm_subscription(
                self.request, subsc_id)
            lccnsubscription['links'] = lccnsubscription.get('_links', '')
            return lccnsubscription
        except Exception as e:
            redirect = reverse(self.redirect_url)
            exceptions.handle(
                self.request,
                _('Failed to get LCCN Subscription. (id: %s)') % subsc_id,
                redirect=redirect)
            raise exceptions.Http302(redirect) from e

    def get_tabs(self, request, *args, **kwargs):
        lccnsubscription = self.get_data()
        return self.tab_group_class(
            request, lccnsubscription=lccnsubscription, **kwargs)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv import utils  # noqa
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfpackages \
    import tables


class VnfPackageItem(object):
    def __init__(self, vnfpkg_id, vnf_product_name, onboarding_state,
                 usage_state, operational_state, vnfd_id,):
        self.id = vnfpkg_id
        self.name = vnfpkg_id
        self.vnf_product_name = vnf_product_name
        self.onboarding_state = onboarding_state
        self.usage_state = usage_state
        self.operational_state = operational_state
        self.vnfd_id = vnfd_id


class VnfPackageTab(tabs.TableTab):
    name = _("VNFP tab")
    slug = "vnfpkg_tab"
    table_classes = (tables.VnfPackageTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_prev_data(self, table):
        return self._has_prev

    def has_more_data(self, table):
        return self._has_more

    def get_vnfpackage_data(self):
        try:
            self._has_prev = False
            page_size = horizon_utils.functions.get_page_size(self.request)
            marker = self.request.GET.get(
                "package_marker", None)
            prev_marker = self.request.GET.get(
                "prev_package_marker", None)
            packages = api.tacker.list_vnf_packages(self.request)
            if marker is not None or prev_marker is not None:
                for i, package in enumerate(packages):
                    if package["id"] == marker and i < len(packages) - 1:
                        packages = packages[i + 1:]
                        self._has_prev = True
                        break
                    if package["id"] == prev_marker and i > page_size:
                        packages = packages[i - page_size:]
                        self._has_prev = True
                        break

            if len(packages) > page_size:
                self._has_more = True
            else:
                self._has_more = False

            rows = []
            for i, package in enumerate(packages):
                if i >= page_size:
                    break
                item = VnfPackageItem(
                    vnfpkg_id=package.get('id', ''),
                    vnf_product_name=package.get('vnfProductName', ''),
                    onboarding_state=package.get('onboardingState', ''),
                    usage_state=package.get('usageState', ''),
                    operational_state=package.get('operationalState', ''),
                    vnfd_id=package.get('vnfdId', ''),)
                rows.append(item)
            return rows
        except Exception:
            self._has_more = False
            error_message = _('Failed to get VNF Packages.')
            exceptions.handle(self.request, error_message)

            return []


class VnfPackageTabs(tabs.TabGroup):
    slug = "vnfpkg_tabs"
    tabs = (VnfPackageTab,)
    sticky = True


class VnfPackageDetailTab(tabs.Tab):
    name = _("VNFPkg Detail")
    slug = "vnfpkg_detail_tab"
    template_name = "nfv/vnfpackages/vnfpkg_detail.html"

    def get_context_data(self, request):
        return {'vnfpkg': self.tab_group.kwargs['vnfpkg']}


class VnfPackageDetailTabs(tabs.TabGroup):
    slug = "vnfpkg_detail_tabs"
    tabs = (VnfPackageDetailTab,)
    sticky = True



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _

from horizon import tables


class UpdateVnfFmAlarm(tables.LinkAction):
    name = "updatealarm"
    verbose_name = _("Update Alarm")
    url = "horizon:nfv:vnffmalarm:updatealarm"
    classes = ("ajax-modal",)


class VnfFmAlarmTable(tables.DataTable):
    id = tables.Column('id', link="horizon:nfv:vnffmalarm:detail",
                       verbose_name=_("ID"))

    managed_object_id = tables.Column('managed_object_id',
                                      verbose_name=_("Managed Object ID"))
    ack_state = tables.Column('ack_state', verbose_name=_("Ack State"))
    event_type = tables.Column('event_type', verbose_name=_("Event Type"))
    perceived_severity = tables.Column('perceived_severity',
                                       verbose_name=_("Perceived Severity"))
    probable_cause = tables.Column('probable_cause',
                                   verbose_name=_("Probable Cause"))

    class Meta(object):
        name = "alarm"
        verbose_name = _("Alarm")
        pagination_param = 'vnffmalarm_marker'
        prev_pagination_param = 'prev_vnffmalarm_marker'
        table_actions = (tables.FilterAction,)
        row_actions = (UpdateVnfFmAlarm,)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from django.core.files.uploadedfile import InMemoryUploadedFile
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.files.uploadedfile import TemporaryUploadedFile
from django.forms import ValidationError
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from oslo_log import log as logging

from tacker_horizon.openstack_dashboard import api

LOG = logging.getLogger(__name__)


class UploadVnfPackage(forms.SelfHandlingForm):
    user_data = forms.JSONField(
        label=_("User Data"), required=False)
    source_type = forms.ChoiceField(
        label=_('VNF Package Source'),
        choices=[('url', _('URL')), ('file', _('File'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'source'}))
    url = forms.URLField(
        widget=forms.TextInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-url': _('URL')}),
        required=False)
    user_name = forms.CharField(
        widget=forms.TextInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-url': _('User Name')}),
        required=False)
    password = forms.CharField(
        widget=forms.PasswordInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-url': _('Password')}),
        required=False)
    vnf_package = forms.FileField(
        widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-file': _('VNF Package')}),
        required=False)

    def __init__(self, request, *args, **kwargs):
        super(UploadVnfPackage, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(UploadVnfPackage, self).clean()

        return data

    def handle(self, request, data):
        try:
            body = {}
            user_data = data.get('user_data', None)
            if user_data:
                body['userDefinedData'] = user_data
            response = api.tacker.create_vnf_package(request, body)
        except Exception:
            exceptions.handle(request,
                              _('Failed to create VNF Package.'))
            return False

        try:
            params = {}
            source_type = data.get('source_type', None)
            if source_type == 'url':
                params['url'] = data.get('url', None)
                user_name = data.get('user_name', None)
                password = data.get('password', None)
                if user_name:
                    params['userName'] = user_name
                    params['password'] = password
                api.tacker.upload_vnf_package(
                    request, response['id'], **params)
                messages.success(
                        request,
                        _('Accepted to upload VNF Package. '
                          '(id: %s)') % response['id'])
            else:
                file = request.FILES['vnf_package']
                if isinstance(file, TemporaryUploadedFile):
                    # Hack to fool Django, so we can keep file open.
                    file.file._closer.close_called = True
                elif isinstance(file, InMemoryUploadedFile):
                    # Clone a new file for InMemoryUploadedFile.
                    # Because the old one will be closed by Django.
                    file = SimpleUploadedFile(
                        file.name, file.read(), file.content_type)
                try:
                    api.tacker.upload_vnf_package(
                        request, response['id'], file, **params)
                    messages.success(
                        request,
                        _('Accepted to upload VNF Package. '
                          '(id: %s)') % response['id'])
                finally:
                    try:
                        filename = str(file.file.name)
                    except AttributeError:
                        pass
                    else:
                        try:
                            os.remove(filename)
                        except OSError as e:
                            LOG.warning(
                                'Failed to remove temporary file '
                                '%(file)s (%(e)s)',
                                {'file': filename, 'e': e})

        except Exception:
            exceptions.handle(request,
                              _('Failed to upload VNF Package. (id: %s)') %
                              response['id'])
            return False

        return True


class UpdateVnfPackage(forms.SelfHandlingForm):
    operationalState = forms.ChoiceField(
        label=_('Operational State'),
        required=False,
        choices=[(None, _(' ')),
                 ('ENABLED', _('ENABLED')),
                 ('DISABLED', _('DISABLED'))],
        widget=forms.Select(
            attrs={'class': 'switchable', 'data-slug': 'source'}))
    userData = forms.JSONField(label=_('User Data'),
                               required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateVnfPackage, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(UpdateVnfPackage, self).clean()
        operationalState = data.get('operationalState', None)
        userData = data.get('userData', None)
        if operationalState is None and userData is None:
            raise ValidationError(
                _("At least one of the \"operationalState\" or \
                  \"userDefinedData\" parameters shall be present."))
        return data

    def handle(self, request, data):
        try:
            vnfpkg_id = request.resolver_match.kwargs.get('id', None)
            body = {}
            operationalState = data.get('operationalState', None)
            if operationalState:
                body['operationalState'] = operationalState

            userDefinedData = data.get('userData', None)
            if userDefinedData:
                body['userDefinedData'] = userDefinedData
            api.tacker.update_vnf_package(request, vnfpkg_id, body)
            messages.success(request,
                             _('Update VNF Package. (id: %s)') % vnfpkg_id)

        except Exception:
            exceptions.handle(request,
                              _('Failed to update VNF Package. (id: %s)') %
                              vnfpkg_id)
            return False

        return True



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffmalarm \
    import forms as project_forms
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffmalarm \
    import tabs as alarm_tabs


class IndexView(tabs.TabbedTableView):
    # A very simple class-based view...
    tab_group_class = alarm_tabs.VnfFmAlarmTabs
    template_name = 'nfv/vnffmalarm/index.html'

    def get_data(self, request, context, *args, **kwargs):
        # Add data to the context here...
        return context


class UpdateVnfFmAlarmView(forms.ModalFormView):
    form_class = project_forms.UpdateVnfFmAlarm
    template_name = 'nfv/vnffmalarm/update_alarm.html'
    success_url = reverse_lazy("horizon:nfv:vnffmalarm:index")
    modal_id = "update_alarm_modal"
    modal_header = _("Update Alarm")
    submit_label = _("Update Alarm")
    submit_url = "horizon:nfv:vnffmalarm:updatealarm"

    def get_context_data(self, **kwargs):
        context = super(UpdateVnfFmAlarmView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class DetailView(tabs.TabView):
    tab_group_class = alarm_tabs.VnfFmAlarmDetailTabs
    template_name = 'nfv/vnffmalarm/detail.html'
    redirect_url = 'horizon:nfv:vnffmalarm:index'
    page_title = _("Alarm Detail: {{ id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        alarm = self.get_data()
        context['alarm'] = alarm
        context['id'] = kwargs['id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        alarm_id = self.kwargs['id']

        try:
            alarm = tacker_api.tacker.get_fm_alarm(self.request, alarm_id)
            alarm['links'] = alarm.get('_links', '')
            return alarm
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Failed to get FM Alarm. (id: %s)') % alarm_id,
                              redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        alarm = self.get_data()
        return self.tab_group_class(request, alarm=alarm, **kwargs)



# Copyright 2016 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _

from horizon import tables


class EventItem(object):
    def __init__(self, id, state, type, timestamp, details):
        self.id = id
        self.resource_state = state
        self.event_type = type
        self.timestamp = timestamp
        self.event_details = details


class EventItemList(object):
    EVTLIST_P = []

    @classmethod
    def add_item(cls, item):
        cls.EVTLIST_P.append(item)

    @classmethod
    def clear_list(cls):
        cls.EVTLIST_P = []


class EventsTable(tables.DataTable):

    id = tables.Column('id', verbose_name=_("Event ID"))
    resource_state = tables.Column('resource_state',
                                   verbose_name=_("Resource State"))
    timestamp = tables.Column('timestamp',
                              verbose_name=_("Time Since Event"))
    event_type = tables.Column("event_type", verbose_name=_("Event Type"))
    event_details = tables.Column("event_details",
                                  verbose_name=_("Event Details"))

    class Meta(object):
        name = "events"



# Copyright 2016 Brocade Communications System, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import tables

from openstack_dashboard import policy
from tacker_horizon.openstack_dashboard import api


class MyFilterAction(tables.FilterAction):
    name = "myfilter"


class DeleteVIMLink(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete VIM",
            "Delete VIMs",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Delete VIM",
            "Delete VIMs",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_vim(request, obj_id)


class RegisterVIMLink(tables.LinkAction):
    name = "registervim"
    verbose_name = _("Register VIM")
    classes = ("ajax-modal",)
    icon = "plus"
    url = "horizon:nfv:vim:registervim"


class VIMTable(tables.DataTable):
    name = tables.Column('name', verbose_name=_("Name"),
                         link="horizon:nfv:vim:detail",)
    description = tables.Column('description', verbose_name=_("Description"))
    id = tables.Column('id', verbose_name=_("VIM Id"))
    is_default = tables.Column('is_default', verbose_name=_("Default"))
    auth_url = tables.Column('auth_url', verbose_name=_("Auth URL"))
    regions = tables.Column('regions', verbose_name=_("Regions"))
    user = tables.Column('user', verbose_name=_("User"))
    project = tables.Column('project', verbose_name=_("Project"))
    status = tables.Column('status', verbose_name=_("Status"))
    type = tables.Column('type', verbose_name=_("VIM Type"))

    class Meta(object):
        name = "vim"
        verbose_name = _("VIM")
        table_actions = (RegisterVIMLink, DeleteVIMLink, MyFilterAction,)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack_dashboard.test.integration_tests.pages import basepage


class VnfPackagesPage(basepage.BaseNavigationPage):
    def __init__(self, driver, conf):
        super(VnfPackagesPage, self).__init__(driver, conf)
        self._page_title = "VNF Packages"



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from tacker_horizon.openstack_dashboard import api


class CreatePmJob(forms.SelfHandlingForm):
    param_file = forms.FileField(
        label=_("Param File"),
        help_text=_("parameter file to upload."),
        widget=forms.FileInput(
            attrs={'class': 'switched',
                   'data-switch-on': 'source',
                   'data-source-file': _('Param File')}))

    def __init__(self, request, *args, **kwargs):
        super(CreatePmJob, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(CreatePmJob, self).clean()

        try:
            param_str = self.files['param_file'].read()
            param = json.loads(param_str)
            data['param'] = param
        except Exception as e:
            msg = _('Failed to read file: %s.') % e
            raise forms.ValidationError(msg)

        return data

    def handle(self, request, data):
        try:
            param = data['param']
            pmjob = api.tacker.create_pm_job(request, param)
            messages.success(request,
                             _('Create PM Job. (id: %s)') %
                             pmjob['id'])
            return True
        except Exception:
            msg = _('Failed to create PM Job.')
            exceptions.handle(request, msg)
            return False


class UpdatePmJob(forms.SelfHandlingForm):
    param_file = forms.FileField(
        label=_("Param File"),
        help_text=_("parameter file to upload."),
        widget=forms.FileInput(
            attrs={'class': 'switched',
                   'data-switch-on': 'source',
                   'data-source-file': _('Param File')}))

    def __init__(self, request, *args, **kwargs):
        super(UpdatePmJob, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(UpdatePmJob, self).clean()

        try:
            param_str = self.files['param_file'].read()
            param = json.loads(param_str)
            data['param'] = param
        except Exception as e:
            msg = _('Failed to read file: %s.') % e
            raise forms.ValidationError(msg)

        return data

    def handle(self, request, data):
        try:
            param = data['param']
            job_id = request.resolver_match.kwargs['id']
            api.tacker.update_pm_job(request, job_id, param)
            messages.success(request,
                             _('Update PM Job. (id: %s)') % job_id)
            return True
        except Exception:
            msg = _('Failed to update PM Job. (id: %s)') % job_id
            exceptions.handle(request, msg)
            return False



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnflcm import tables
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnflcmopocc \
    import tabs as opocc_tab


class VnfLcmItem(object):
    def __init__(self, vnf_id, vnf_instance_name, instantiation_state,
                 vnf_provider, vnf_software_version, vnf_product_name,
                 vnfd_id):
        self.id = vnf_id
        self.name = vnf_id
        self.vnf_instance_name = vnf_instance_name
        self.instantiation_state = instantiation_state
        self.vnf_provider = vnf_provider
        self.vnf_software_version = vnf_software_version
        self.vnf_product_name = vnf_product_name
        self.vnfd_id = vnfd_id


class VnfLcmTab(tabs.TableTab):
    name = _("VNFLCM tab")
    slug = "vnflcm_tab"
    table_classes = (tables.VnfLcmTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_prev_data(self, table):
        return self._has_prev

    def has_more_data(self, table):
        return self._has_more

    def get_vnflcm_data(self):
        try:
            self._has_prev = False
            page_size = horizon_utils.functions.get_page_size(self.request)
            marker = self.request.GET.get(
                "vnflcm_marker", None)
            prev_marker = self.request.GET.get(
                "prev_vnflcm_marker", None)
            vnf_instances = api.tacker.list_vnf_instances(self.request)
            if marker is not None or prev_marker is not None:
                for i, instance in enumerate(vnf_instances):
                    if instance["id"] == marker and i < len(vnf_instances) - 1:
                        vnf_instances = vnf_instances[i + 1:]
                        self._has_prev = True
                        break
                    if instance["id"] == prev_marker and i > page_size:
                        vnf_instances = vnf_instances[i - page_size:]
                        self._has_prev = True
                        break

            if len(vnf_instances) > page_size:
                self._has_more = True
            else:
                self._has_more = False

            rows = []
            for i, instance in enumerate(vnf_instances):
                if i >= page_size:
                    break
                item = VnfLcmItem(
                    vnf_id=instance.get('id', ''),
                    vnf_instance_name=instance.get('vnfInstanceName', ''),
                    instantiation_state=instance.get(
                        'instantiationState', ''),
                    vnf_provider=instance.get('vnfProvider', ''),
                    vnf_software_version=instance.get(
                        'vnfSoftwareVersion', ''),
                    vnf_product_name=instance.get('vnfProductName', ''),
                    vnfd_id=instance.get('vnfdId', ''))
                rows.append(item)
            return rows
        except Exception:
            self._has_more = False
            error_message = _('Failed to get VNF Instances.')
            exceptions.handle(self.request, error_message)

            return []


class VnfLcmTabs(tabs.TabGroup):
    slug = "vnflcm_tabs"
    tabs = (VnfLcmTab,)
    sticky = True


class VnfLcmOpOccItem(object):
    def __init__(self, id, operation_state, vnf_instance_id, operation):
        self.id = id
        self.operation_state = operation_state
        self.vnf_instance_id = vnf_instance_id
        self.operation = operation


class VnfLcmDetailTab(tabs.Tab):
    name = _("VNF Instance Detail Tab")
    slug = "vnflcm_detail_tab"
    template_name = "nfv/vnflcm/vnflcm_detail.html"

    def get_context_data(self, request):
        return {'vnflcm': self.tab_group.kwargs['vnflcm']}


class VnfLcmDetailTabs(tabs.TabGroup):
    slug = "vnflcm_detail_tabs"
    tabs = (VnfLcmDetailTab, opocc_tab.VnfLcmOpOccTab,)
    sticky = True



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnflcm \
    import forms as project_forms
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnflcm \
    import tabs as vnflcm_tabs


class IndexView(tabs.TabbedTableView):
    tab_group_class = vnflcm_tabs.VnfLcmTabs
    template_name = 'nfv/vnflcm/index.html'


class CreateVnfIdentifierView(forms.ModalFormView):
    form_class = project_forms.CreateVnfIdentifier
    template_name = 'nfv/vnflcm/create_vnf_identifier.html'
    success_url = reverse_lazy("horizon:nfv:vnflcm:index")
    modal_id = "add_service_modal"
    modal_header = _("Create VNF Identifier")
    submit_label = _("Create VNF Identifier")
    submit_url = "horizon:nfv:vnflcm:createvnfidentifier"

    def get_context_data(self, **kwargs):
        context = super(
            CreateVnfIdentifierView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url)
        return context


class InstantiateVnfView(forms.ModalFormView):
    form_class = project_forms.InstantiateVnf
    template_name = 'nfv/vnflcm/instantiate_vnf.html'
    success_url = reverse_lazy("horizon:nfv:vnflcm:index")
    modal_id = "add_service_modal"
    modal_header = _("Instantiate VNF")
    submit_label = _("Instantiate VNF")
    submit_url = "horizon:nfv:vnflcm:instantiatevnf"

    def get_context_data(self, **kwargs):
        context = super(InstantiateVnfView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class TerminateVnfView(forms.ModalFormView):
    form_class = project_forms.TerminateVnf
    template_name = 'nfv/vnflcm/terminate_vnf.html'
    success_url = reverse_lazy("horizon:nfv:vnflcm:index")
    modal_id = "add_service_modal"
    modal_header = _("Terminate VNF")
    submit_label = _("Terminate VNF")
    submit_url = "horizon:nfv:vnflcm:terminatevnf"

    def get_context_data(self, **kwargs):
        context = super(TerminateVnfView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class HealVnfView(forms.ModalFormView):
    form_class = project_forms.HealVnf
    template_name = 'nfv/vnflcm/heal_vnf.html'
    success_url = reverse_lazy("horizon:nfv:vnflcm:index")
    modal_id = "add_service_modal"
    modal_header = _("Heal VNF")
    submit_label = _("Heal VNF")
    submit_url = "horizon:nfv:vnflcm:healvnf"

    def get_context_data(self, **kwargs):
        context = super(HealVnfView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class UpdateVnfView(forms.ModalFormView):
    form_class = project_forms.UpdateVnf
    template_name = 'nfv/vnflcm/update_vnf.html'
    success_url = reverse_lazy("horizon:nfv:vnflcm:index")
    modal_id = "add_service_modal"
    modal_header = _("Update VNF")
    submit_label = _("Update VNF")
    submit_url = "horizon:nfv:vnflcm:updatevnf"

    def get_context_data(self, **kwargs):
        context = super(UpdateVnfView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class ScaleVnfView(forms.ModalFormView):
    form_class = project_forms.ScaleVnf
    template_name = 'nfv/vnflcm/scale_vnf.html'
    success_url = reverse_lazy("horizon:nfv:vnflcm:index")
    modal_id = "add_service_modal"
    modal_header = _("Scale VNF")
    submit_label = _("Scale VNF")
    submit_url = "horizon:nfv:vnflcm:scalevnf"

    def get_context_data(self, **kwargs):
        context = super(ScaleVnfView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class ChangeExternalVnfConnectivityView(forms.ModalFormView):
    form_class = project_forms.ChangeExternalVnfConnectivity
    template_name = 'nfv/vnflcm/change_connectivity.html'
    success_url = reverse_lazy("horizon:nfv:vnflcm:index")
    modal_id = "add_service_modal"
    modal_header = _("Change External VNF Connectivity")
    submit_label = _("Change External VNF Connectivity")
    submit_url = "horizon:nfv:vnflcm:changeconnectivity"

    def get_context_data(self, **kwargs):
        context = super(
            ChangeExternalVnfConnectivityView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class ChangeCurrentVnfPackageView(forms.ModalFormView):
    form_class = project_forms.ChangeCurrentVnfPackage
    template_name = 'nfv/vnflcm/change_vnfpkg.html'
    success_url = reverse_lazy("horizon:nfv:vnflcm:index")
    modal_id = "add_service_modal"
    modal_header = _("Change Current VNF Package")
    submit_label = _("Change Current VNF Package")
    submit_url = "horizon:nfv:vnflcm:changevnfpkg"

    def get_context_data(self, **kwargs):
        context = super(
            ChangeCurrentVnfPackageView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class DetailView(tabs.TabbedTableView):
    tab_group_class = vnflcm_tabs.VnfLcmDetailTabs
    template_name = 'nfv/vnflcm/detail.html'
    redirect_url = 'horizon:nfv:vnflcm:index'
    page_title = _("VNF Instance Detail: {{ id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        vnflcm = self.get_data()
        context['vnflcm'] = vnflcm
        context['id'] = kwargs['id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        vnf_id = self.kwargs['id']

        try:
            vnflcm = tacker_api.tacker.get_vnf_instance(self.request, vnf_id)
            vnflcm['links'] = vnflcm.get('_links', '')
            return vnflcm
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(
                self.request,
                _('Failed to get VNF instance. (id: %s)') % vnf_id,
                redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        vnflcm = self.get_data()
        return self.tab_group_class(request, vnflcm=vnflcm, **kwargs)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import exceptions
from horizon import messages
from horizon import tables
from openstack_dashboard import policy

from tacker_horizon.openstack_dashboard import api


class DeleteVnfPackage(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete VNF Package",
            "Delete VNF Packages",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Delete VNF Package",
            "Delete VNF Packages",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_vnf_package(request, obj_id)


class UploadVnfPackage(tables.LinkAction):
    name = "uploadvnfpkg"
    verbose_name = _("Upload VNF Package")
    classes = ("ajax-modal",)
    icon = "plus"
    url = "horizon:nfv:vnfpackages:uploadvnfpkg"


class FetchVnfPackage(tables.LinkAction):
    name = "fetchvnfpkg"
    verbose_name = _("Fetch VNF Package")
    verbose_name_plural = _("Fetch VNF Package")
    icon = "download"
    url = "horizon:nfv:vnfpackages:fetch"

    def allowed(self, request, datum=None):
        return True


class UpdateVnfPackageInfo(tables.LinkAction):
    name = "updatevnfpkginfo"
    verbose_name = _("Update VNF Package Info")
    url = "horizon:nfv:vnfpackages:updatevnfpkg"
    classes = ("ajax-modal",)


class CreateVnfIdentifier(tables.Action):
    name = "createvnfid"
    verbose_name = _("Create VNF Identifier")

    def single(self, table, request, obj_id):  # pylint: disable=E0202
        try:
            vnf_pkg = api.tacker.get_vnf_package(request, obj_id)
            vnfd_id = vnf_pkg.get('vnfdId', None)
            if not vnfd_id:
                msg = _('Failed to get VNFD ID')
                raise Exception(msg)
            req_body = {}
            req_body['vnfdId'] = vnfd_id
            response = api.tacker.create_vnf_instance(request, req_body)
            messages.success(request,
                             _('Create VNF Identifier. (id: %s)') %
                             response['id'])
        except Exception:
            exceptions.handle(request,
                              _('Failed to create VNF Identifier.'))


class VnfPackageTable(tables.DataTable):
    id = tables.Column('id', verbose_name=_("ID"),
                       link="horizon:nfv:vnfpackages:detail",)
    name = tables.Column('vnf_product_name',
                         verbose_name=_("VNF Product Name"))
    onboarding_state = tables.Column('onboarding_state',
                                     verbose_name=_("Onboarding State"))
    usage_state = tables.Column('usage_state', verbose_name=_("Usage State"))
    operational_state = tables.Column('operational_state',
                                      verbose_name=_("Operational State"))

    class Meta(object):
        name = "vnfpackage"
        verbose_name = _("VNF Package")
        pagination_param = 'package_marker'
        prev_pagination_param = 'prev_package_marker'
        table_actions = (UploadVnfPackage, DeleteVnfPackage,
                         tables.FilterAction,)
        row_actions = (FetchVnfPackage, UpdateVnfPackageInfo, DeleteVnfPackage,
                       CreateVnfIdentifier,)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfpmthreshold \
    import forms as project_forms
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnfpmthreshold \
    import tabs as pmthreshold_tabs


class IndexView(tabs.TabbedTableView):
    # A very simple class-based view...
    tab_group_class = pmthreshold_tabs.VnfPmThresholdTabs
    template_name = 'nfv/vnfpmthreshold/index.html'

    def get_data(self, request, context, *args, **kwargs):
        # Add data to the context here...
        return context


class CreatePmThresholdView(forms.ModalFormView):
    form_class = project_forms.CreatePmThreshold
    template_name = 'nfv/vnfpmthreshold/create_pmthreshold.html'
    success_url = reverse_lazy("horizon:nfv:vnfpmthreshold:index")
    modal_id = "create_pmthreshold_modal"
    modal_header = _("Create PM Threshold")
    submit_label = _("Create PM Threshold")
    submit_url = "horizon:nfv:vnfpmthreshold:createpmthreshold"

    def get_context_data(self, **kwargs):
        context = super(CreatePmThresholdView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url)
        return context


class UpdatePmThresholdView(forms.ModalFormView):
    form_class = project_forms.UpdatePmThreshold
    template_name = 'nfv/vnfpmthreshold/update_pmthreshold.html'
    success_url = reverse_lazy("horizon:nfv:vnfpmthreshold:index")
    modal_id = "update_pmthreshold_modal"
    modal_header = _("Update PM Threshold")
    submit_label = _("Update PM Threshold")
    submit_url = "horizon:nfv:vnfpmthreshold:updatepmthreshold"

    def get_context_data(self, **kwargs):
        context = super(UpdatePmThresholdView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url,
                                        kwargs={'id': self.kwargs['id']})
        return context


class DetailView(tabs.TabView):
    tab_group_class = pmthreshold_tabs.VnfPmThresholdDetailTabs
    template_name = 'nfv/vnfpmthreshold/detail.html'
    redirect_url = 'horizon:nfv:vnfpmthreshold:index'
    page_title = _("PM Threshold Detail: {{ id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        pmthreshold = self.get_data()
        context['pmthreshold'] = pmthreshold
        context['id'] = kwargs['id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        threshold_id = self.kwargs['id']

        try:
            pmthreshold = tacker_api.tacker.get_pm_threshold(self.request,
                                                             threshold_id)
            pmthreshold['links'] = pmthreshold.get('_links', '')
            pmthreshold['callback_uri'] = pmthreshold.get('callbackUri', '')
            pmthreshold['criteria'] = pmthreshold.get('criteria', '')
            pmthreshold['id'] = pmthreshold.get('id', '')
            pmthreshold['object_instance_id'] = pmthreshold.get(
                                                    'objectInstanceId', '')
            pmthreshold['object_type'] = pmthreshold.get('objectType', '')
            pmthreshold['sub_object_instance_ids'] = pmthreshold.get(
                                                'subObjectInstanceIds', '')
            return pmthreshold
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Failed to get PM Threshold. (id: %s)') %
                              threshold_id, redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        pmthreshold = self.get_data()
        return self.tab_group_class(request, pmthreshold=pmthreshold, **kwargs)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import tables
from openstack_dashboard import policy

from tacker_horizon.openstack_dashboard import api


class DeleteVnfIdentifier(policy.PolicyTargetMixin, tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Delete VNF Identifier",
            "Delete VNF Identifiers",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Delete VNF Identifier",
            "Delete VNF Identifiers",
            count
        )

    def action(self, request, obj_id):
        api.tacker.delete_vnf_instance(request, obj_id)


class CreateVnfIdentifier(tables.LinkAction):
    name = "createvnfid"
    verbose_name = _("Create VNF Identifier")
    url = "horizon:nfv:vnflcm:createvnfidentifier"
    classes = ("ajax-modal",)
    icon = "plus"


class InstantiateVnf(tables.LinkAction):
    name = "instantiatevnf"
    verbose_name = _("Instantiate VNF")
    url = "horizon:nfv:vnflcm:instantiatevnf"
    classes = ("ajax-modal",)


class TerminateVnf(tables.LinkAction):
    name = "terminate_vnf"
    verbose_name = _("Terminate VNF")
    url = "horizon:nfv:vnflcm:terminatevnf"
    classes = ("ajax-modal",)


class HealVnf(tables.LinkAction):
    name = "heal_vnf"
    verbose_name = _("Heal VNF")
    url = "horizon:nfv:vnflcm:healvnf"
    classes = ("ajax-modal",)


class UpdateVnf(tables.LinkAction):
    name = "update_Vnf"
    verbose_name = _("Update VNF")
    url = "horizon:nfv:vnflcm:updatevnf"
    classes = ("ajax-modal",)


class ScaleVnf(tables.LinkAction):
    name = "scale_vnf"
    verbose_name = _("Scale VNF")
    url = "horizon:nfv:vnflcm:scalevnf"
    classes = ("ajax-modal",)


class ChangeExternalVnfConnectivity(tables.LinkAction):
    name = "change_external_vnf_connectivity"
    verbose_name = _("Change External VNF Connectivity")
    url = "horizon:nfv:vnflcm:changeconnectivity"
    classes = ("ajax-modal",)


class ChangeCurrentVnfPackage(tables.LinkAction):
    name = "change_current_vnf_package"
    verbose_name = _("Change Current VNF Package")
    url = "horizon:nfv:vnflcm:changevnfpkg"
    classes = ("ajax-modal",)


class VnfLcmTable(tables.DataTable):
    id = tables.Column('id', verbose_name=_("ID"),
                       link="horizon:nfv:vnflcm:detail",)
    vnf_instance_name = tables.Column('vnf_instance_name',
                                      verbose_name=_("VNF Instance Name"))
    instantiation_state = tables.Column('instantiation_state',
                                        verbose_name=_("Instantiation State"))
    vnf_provider = tables.Column('vnf_provider',
                                 verbose_name=_("VNF Provider"))
    vnf_software_version = tables.Column(
        'vnf_software_version',
        verbose_name=_("VNF Software Version"))
    vnf_product_name = tables.Column('vnf_product_name',
                                     verbose_name=_("VNF Product Name"))
    vnfd_id = tables.Column('vnfd_id', verbose_name=_("VNFD ID"))

    class Meta(object):
        name = "vnflcm"
        verbose_name = _("VNF LCM")
        pagination_param = 'vnflcm_marker'
        prev_pagination_param = 'prev_vnflcm_marker'
        table_actions = (CreateVnfIdentifier, DeleteVnfIdentifier,
                         tables.FilterAction,)
        row_actions = (InstantiateVnf, TerminateVnf, DeleteVnfIdentifier,
                       HealVnf, UpdateVnf, ScaleVnf,
                       ChangeExternalVnfConnectivity, ChangeCurrentVnfPackage,)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffmsubscription \
    import forms as project_forms
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnffmsubscription \
    import tabs as subscription_tabs


class IndexView(tabs.TabbedTableView):
    # A very simple class-based view...
    tab_group_class = subscription_tabs.VnfFmSubscriptionTabs
    template_name = 'nfv/vnffmsubscription/index.html'

    def get_data(self, request, context, *args, **kwargs):
        # Add data to the context here...
        return context


class CreateVnfFmSubscriptionView(forms.ModalFormView):
    form_class = project_forms.CreateVnfFmSubscription
    template_name = 'nfv/vnffmsubscription/create_subscription.html'
    success_url = reverse_lazy("horizon:nfv:vnffmsubscription:index")
    modal_id = "create_subscription_modal"
    modal_header = _("Create Subscription")
    submit_label = _("Create Subscription")
    submit_url = "horizon:nfv:vnffmsubscription:createsubscription"

    def get_context_data(self, **kwargs):
        context = super(CreateVnfFmSubscriptionView,
                        self).get_context_data(**kwargs)
        context['submit_url'] = reverse(self.submit_url)
        return context


class DetailView(tabs.TabView):
    tab_group_class = subscription_tabs.VnfFmSubscriptionDetailTabs
    template_name = 'nfv/vnffmsubscription/detail.html'
    redirect_url = 'horizon:nfv:vnffmsubscription:index'
    page_title = _("Subscription Detail: {{ id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        subscription = self.get_data()
        context['subscription'] = subscription
        context['id'] = kwargs['id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        subscription_id = self.kwargs['id']

        try:
            subscription = tacker_api.tacker.get_fm_subscription(
                self.request, subscription_id)
            subscription['links'] = subscription.get('_links', '')
            subscription['callback_uri'] = subscription.get('callbackUri', '')
            subscription['filter'] = subscription.get('filter', '')
            return subscription
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(self.request,
                              _('Failed to get FM Subscription. (id: %s)') %
                              subscription_id, redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        subscription = self.get_data()
        return self.tab_group_class(request,
                                    subscription=subscription, **kwargs)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon.utils import memoized

from tacker_horizon.openstack_dashboard import api as tacker_api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnflcmopocc \
    import tabs as vnflcmopocc_tabs


class IndexView(tabs.TabbedTableView):
    # A very simple class-based view...
    tab_group_class = vnflcmopocc_tabs.VnfLcmOpOccTabs
    template_name = 'nfv/vnflcmopocc/index.html'

    def get_data(self, request, context, *args, **kwargs):
        # Add data to the context here...
        return context


class DetailView(tabs.TabView):
    tab_group_class = vnflcmopocc_tabs.VnfOpOccDetailTabs
    template_name = 'nfv/vnflcmopocc/detail.html'
    redirect_url = 'horizon:nfv:vnflcmopocc:index'
    page_title = _("LCM OP OCC Details: {{ id }}")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        vnflcmopocc = self.get_data()
        context['vnflcmopocc'] = vnflcmopocc
        context['id'] = kwargs['id']
        context['url'] = reverse(self.redirect_url)
        return context

    @memoized.memoized_method
    def get_data(self):
        opocc_id = self.kwargs['id']

        try:
            vnflcmopocc = tacker_api.tacker.get_vnf_lcm_op_occ(self.request,
                                                               opocc_id)
            vnflcmopocc['links'] = vnflcmopocc.get('_links', '')
            return vnflcmopocc
        except Exception:
            redirect = reverse(self.redirect_url)
            exceptions.handle(
                self.request,
                _('Failed to get VNF LCM operation occurrence. (id: %s)') %
                opocc_id,
                redirect=redirect)
            raise exceptions.Http302(redirect)

    def get_tabs(self, request, *args, **kwargs):
        vnflcmopocc = self.get_data()
        return self.tab_group_class(request, vnflcmopocc=vnflcmopocc, **kwargs)



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from tacker_horizon.openstack_dashboard import api


class UpdateVnfFmAlarm(forms.SelfHandlingForm):
    ack_state = forms.ChoiceField(
        label=_('ACK State'),
        choices=[('ACKNOWLEDGED', _('ACKNOWLEDGED')),
                 ('UNACKNOWLEDGED', _('UNACKNOWLEDGED'))])

    def __init__(self, request, *args, **kwargs):
        super(UpdateVnfFmAlarm, self).__init__(request, *args, **kwargs)

    def clean(self):
        data = super(UpdateVnfFmAlarm, self).clean()
        return data

    def handle(self, request, data):
        try:
            ack_state = data.get('ack_state', None)

            body = {}
            body['ackState'] = ack_state
            alarm_id = request.resolver_match.kwargs['id']
            api.tacker.update_fm_alarm(request, alarm_id, body)
            messages.success(request,
                             _('Update FM Alarm. (id: %s)') % alarm_id)
            return True
        except Exception:
            msg = _('Failed to update FM Alarm. (id: %s)') % alarm_id
            exceptions.handle(request, msg)
            return False



# Copyright (C) 2024 Fujitsu
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tabs
from horizon import utils as horizon_utils

from tacker_horizon.openstack_dashboard import api
from tacker_horizon.openstack_dashboard.dashboards.nfv.vnflcmopocc \
    import tables


class VnfLcmOpOccItem(object):
    def __init__(self, opocc_id, operation_state, vnf_instance_id,
                 operation):
        self.id = opocc_id
        self.name = opocc_id
        self.operation_state = operation_state
        self.vnf_instance_id = vnf_instance_id
        self.operation = operation


class VnfLcmOpOccTab(tabs.TableTab):
    name = _("List LCM Operation Occurrences")
    slug = "vnflcmopocc_tab"
    table_classes = (tables.VnfLcmOpOccTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = False

    def has_prev_data(self, table):
        return self._has_prev

    def has_more_data(self, table):
        return self._has_more

    def get_vnflcmopocc_data(self):
        try:
            opoccs = []
            self._has_prev = False
            page_size = horizon_utils.functions.get_page_size(self.request)
            marker = self.request.GET.get(
                "opocc_marker", None)
            prev_marker = self.request.GET.get(
                "prev_opocc_marker", None)
            params = {}
            opocc_id = self.request.resolver_match.kwargs.get("id", None)
            if opocc_id:
                params["filter"] = "(eq,vnfInstanceId,{0})".format(opocc_id)
            opoccs = api.tacker.list_vnf_lcm_op_occs(self.request, **params)
            if marker is not None or prev_marker is not None:
                for i, opocc in enumerate(opoccs):
                    if opocc["id"] == marker and i < len(opoccs) - 1:
                        opoccs = opoccs[i + 1:]
                        self._has_prev = True
                        break
                    if opocc["id"] == prev_marker and i > page_size:
                        opoccs = opoccs[i - page_size:]
                        self._has_prev = True
                        break

            if len(opoccs) > page_size:
                self._has_more = True
            else:
                self._has_more = False

            rows = []
            for i, opocc in enumerate(opoccs):
                if i >= page_size:
                    break
                item = VnfLcmOpOccItem(
                    opocc_id=opocc.get('id', ''),
                    operation_state=opocc.get('operationState', ''),
                    vnf_instance_id=opocc.get('vnfInstanceId', ''),
                    operation=opocc.get('operation', ''))
                rows.append(item)
            return rows
        except Exception:
            self._has_more = False
            error_message = _('Failed to get VNF LCM operation occurrences.')
            exceptions.handle(self.request, error_message)

            return []


class VnfLcmOpOccTabs(tabs.TabGroup):
    slug = "vnflcmopocc_tabs"
    tabs = (VnfLcmOpOccTab,)
    sticky = True


class VnfOpOccDetailTab(tabs.Tab):
    name = _("LCM OP OCC Detail")
    slug = "vnflcmopocc_detail_tab"
    template_name = "nfv/vnflcmopocc/vnflcmopocc_detail.html"

    def get_context_data(self, request):
        return {'vnflcmopocc': self.tab_group.kwargs['vnflcmopocc']}


class VnfOpOccDetailTabs(tabs.TabGroup):
    slug = "vnflcmopocc_detail_tabs"
    tabs = (VnfOpOccDetailTab,)
    sticky = True
